"""The Color module for labyr"""

# TODO: change from using colorama to use rgb(r, g, b)(string)
__all__ = ["C"]

from colorama import Fore, Style, init


class C:
    def __init__(self) -> None:
        init(autoreset=True)  # Ensures colors reset automatically after each print

    # Foreground colors
    @staticmethod
    def red(text: str) -> str:
        return f"{Fore.RED}{text}{Style.RESET_ALL}"

    @staticmethod
    def green(text: str) -> str:
        return f"{Fore.GREEN}{text}{Style.RESET_ALL}"

    @staticmethod
    def brown(text: str) -> str:
        return f"{Fore.YELLOW}{text}{Style.RESET_ALL}"  # Brown is often represented as yellow

    @staticmethod
    def blue(text: str) -> str:
        return f"{Fore.BLUE}{text}{Style.RESET_ALL}"

    @staticmethod
    def purple(text: str) -> str:
        return f"{Fore.MAGENTA}{text}{Style.RESET_ALL}"

    @staticmethod
    def cyan(text: str) -> str:
        return f"{Fore.CYAN}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_gray(text: str) -> str:
        return f"{Fore.LIGHTBLACK_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def dark_gray(text: str) -> str:
        return f"{Fore.LIGHTBLACK_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_red(text: str) -> str:
        return f"{Fore.LIGHTRED_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_green(text: str) -> str:
        return f"{Fore.LIGHTGREEN_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_blue(text: str) -> str:
        return f"{Fore.LIGHTBLUE_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_purple(text: str) -> str:
        return f"{Fore.LIGHTMAGENTA_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_cyan(text: str) -> str:
        return f"{Fore.LIGHTCYAN_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def light_white(text: str) -> str:
        return f"{Fore.LIGHTWHITE_EX}{text}{Style.RESET_ALL}"

    @staticmethod
    def yellow(text: str) -> str:
        return f"{Fore.YELLOW}{text}{Style.RESET_ALL}"

    # Text styles
    @staticmethod
    def bold(text: str) -> str:
        return f"{Style.BRIGHT}{text}{Style.RESET_ALL}"

    @staticmethod
    def faint(text: str) -> str:
        return f"{Style.DIM}{text}{Style.RESET_ALL}"

    @staticmethod
    def italic(text: str) -> str:
        return f"\033[3m{text}\033[0m"  # ANSI escape code for italic

    @staticmethod
    def underline(text: str) -> str:
        return f"\033[4m{text}\033[0m"  # ANSI escape code for underline

    @staticmethod
    def blink(text: str) -> str:
        return f"\033[5m{text}\033[0m"  # ANSI escape code for blink

    @staticmethod
    def negative(text: str) -> str:
        return f"\033[7m{text}\033[0m"  # ANSI escape code for negative

    @staticmethod
    def crossed(text: str) -> str:
        return f"\033[9m{text}\033[0m"  # ANSI escape code for crossed-out text


# Handle non-TTY environments and Windows VT mode
if not __import__("sys").stdout.isatty():
    for name in dir(C):
        if not name.startswith("_") and callable(getattr(C, name)):
            setattr(C, name, lambda text: text)  # Replace methods with no-op lambdas
else:
    if __import__("platform").system() == "Windows":
        kernel32 = __import__("ctypes").windll.kernel32
        kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
        del kernel32


# class Colors:
#     """ANSI color codes"""
#
#     BLACK = "\033[0;30m"
#     RED = "\033[0;31m"
#     GREEN = "\033[0;32m"
#     BROWN = "\033[0;33m"
#     BLUE = "\033[0;34m"
#     PURPLE = "\033[0;35m"
#     CYAN = "\033[0;36m"
#     LIGHT_GRAY = "\033[0;37m"
#     DARK_GRAY = "\033[1;30m"
#     LIGHT_RED = "\033[1;31m"
#     LIGHT_GREEN = "\033[1;32m"
#     YELLOW = "\033[1;33m"
#     LIGHT_BLUE = "\033[1;34m"
#     LIGHT_PURPLE = "\033[1;35m"
#     LIGHT_CYAN = "\033[1;36m"
#     LIGHT_WHITE = "\033[1;37m"
#     BOLD = "\033[1m"
#     FAINT = "\033[2m"
#     ITALIC = "\033[3m"
#     UNDERLINE = "\033[4m"
#     BLINK = "\033[5m"
#     NEGATIVE = "\033[7m"
#     CROSSED = "\033[9m"
#     END = "\033[0m"
#
#     # cancel SGR codes if we don't write to a terminal
#     if not __import__("sys").stdout.isatty():
#         for _ in dir():
#             if isinstance(_, str) and _[0] != "_":
#                 locals()[_] = ""
#     else:
#         # set Windows console in VT mode
#         if __import__("platform").system() == "Windows":
#             kernel32 = __import__("ctypes").windll.kernel32
#             kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
#             del kernel32



import argparse
import hashlib
import os
from importlib.metadata import PackageNotFoundError, version


def get_version():
    try:
        return version("labyr")
    except PackageNotFoundError:
        return "unknown"


def run_labyr(level=None):
    from .__init__ import labyr

    labyr(level=level)


def hash_secret(value: str) -> str:
    return hashlib.sha256(value.encode()).hexdigest()


def dev_test():
    if (
        os.environ.get("DEV_TEST_KEY", "")
        == "9e0fa8e2ae379dc69e30b77507f50075f6d627487f10275a87e3d9c32ac72d71"
    ):
        print("success authentication")


def main():
    parser = argparse.ArgumentParser(
        description="labyr: a labyrinth game || DISCLAIMER: if you run labyr, each frame will clear the terminal screen."
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"labyr {get_version()}",
        help="Show labyr version",
    )
    parser.add_argument(
        "-l",
        "--level",
        type=int,
        metavar="x",
        help="Set the labyrinth level (number)",
    )
    parser.add_argument("--devtest", action="store_true", help="devtest")
    parser.add_argument(
        "action",
        choices=["run", "nil"],
        nargs="?",
        help="Action to perform. Allowed values: run, nil",
    )
    args = parser.parse_args()
    if args.devtest:
        dev_test()
        return
    if not args.action:
        parser.error("the following arguments are required: action")
    actions = {"run": lambda: run_labyr(level=args.level), "nil": lambda: None}
    actions[args.action]()


if __name__ == "__main__":
    main()



import msvcrt
import os
from os import system

import requests


def box(string: str) -> str:
    raw_lines = string.split("\n")
    stripped_lines = [line.strip() for line in raw_lines]
    if not stripped_lines:
        return "+--+\n|  |\n+--+"
    width = max((len(line) for line in stripped_lines), default=0)
    border = "+" + "-" * (width + 2) + "+"
    boxed_lines = [
        f"| {line.ljust(width)} |" if line else "|" + " " * (width + 2) + "|"
        for line in stripped_lines
    ]
    return f"{border}\n" + "\n".join(boxed_lines) + f"\n{border}"


def deltemp():
    import sys

    for var in tuple(sys._getframe(1).f_locals):
        if var[0] == "_":
            sys._getframe(1).f_locals.pop(var, None)


def getch():
    try:
        ch = msvcrt.getch()
        if ch in (b"\x00", b"\xe0"):  # Special keys (arrows, function keys, etc.)
            msvcrt.getch()  # Consume the next byte
            return ""  # Or return a specific value for special keys
        return ch.decode("utf-8")
    except Exception:
        return ""


def getchar(d, arg):
    return d.get(arg, d["DEFAULTS"][arg])


def clsscr(*args):
    system("cls" if os.name == "nt" else "clear")


def loadstring(code=None, url=None):
    """
    Execute a string of Python code or code fetched from a URL, similar to Lua's loadstring().

    Args:
        code (str, optional): A string containing Python code to execute.
        url (str, optional): A URL pointing to a text file containing Python code.

    Returns:
        dict: The globals dictionary containing variables/functions defined during execution.

    Raises:
        ValueError: If neither code nor url is provided, or if both are provided.
        Exception: For errors during code execution or URL fetching.
    """
    if (code is None and url is None) or (code is not None and url is not None):
        raise ValueError("Provide either a code string or a URL, not both or neither.")

    # Initialize an empty globals dictionary to avoid polluting the main namespace
    globals_dict = {}

    try:
        if url:
            # Fetch code from the provided URL
            response = requests.get(url)
            response.raise_for_status()  # Raise an exception for bad status codes
            code = response.text

        # Execute the code in the globals dictionary
        exec(code, globals_dict)

        return globals_dict

    except requests.RequestException as e:
        raise Exception(f"Failed to fetch code from URL: {str(e)}")
    except SyntaxError as e:
        raise Exception(f"Syntax error in code: {str(e)}")
    except Exception as e:
        raise Exception(f"Error executing code: {str(e)}")



from sys import exit as _exit
from time import sleep as tsleep
from typing import Any

from ._secret import Secret
from .utils import box, clsscr, color, getch, getchar
from .utils.dot import Dot
from .utils.entity import Actiwall, EntMan, Player, handlemove
from .utils.fill import fill_rect
from .utils.transform import get_map

__all__ = ["labyr", "LabyrGame"]


def genlvls(
    chars: dict,
    lvlSizes: dict[int, tuple[int, int]],
):
    out = {}

    def init(dimen: tuple[int, int]) -> list[list[str]]:
        """generate walls"""
        space = getchar(chars, "space")[0]
        x, y = dimen
        grid = [[space] * x for _ in range(y)]

        for i in range(x):
            for j in range(y):
                if i == 0 or i == x - 1 or j == 0 or j == y - 1:
                    grid[j][i] = "#"

        return grid

    for lvl, dimen in lvlSizes.items():
        out[lvl] = init(dimen)

    space_char = getchar(chars, "space")[0]
    wall_char = getchar(chars, "wall")[0]
    actiwall_char = getchar(chars, "actiwall")[0]
    player_char = getchar(chars, "player")[0]
    exit_char = getchar(chars, "exit")[0]

    def set_chars(out, lvl, positions, char, **kwds):
        actiwall = kwds.get("actiwall")
        if actiwall:
            value = (
                char,
                Actiwall(
                    actiwall["pos"],
                    actiwall["goto"],
                    actiwall["plate"],
                    actiwall["retplate"],
                ),
            )
        else:
            value = char

        # Normalize positions to a list of tuples
        if (
            isinstance(positions, tuple)
            and len(positions) == 2
            and all(isinstance(i, int) for i in positions)
        ):
            positions = [positions]

        for y, x in positions:
            out[lvl][y][x] = value

    def rect(y1: int, x1: int, y2: int, x2: int):
        return ((y1, x1), (y2, x2))

    def srect(y: int, x: int):
        return ((y, x), (y, x))

    level_defs = [
        # lvl 0 conf
        # #######
        # #@...E#
        # #######
        {"player": [(1, 1)], "exit": [(1, 5)], "walls": [], "spaces": []},
        # lvl 1 conf
        # ###########
        # #@......#E#
        # #######.#.#
        # #.......#.#
        # #.#######.#
        # #.........#
        # ###########
        {
            "player": [(1, 1)],
            "exit": [(1, -2)],
            "walls": [
                rect(2, 1, 2, 6),
                rect(4, 2, 4, -3),
                rect(1, -3, 3, -3),
            ],
            "spaces": [],
        },
        # lvl 2 conf
        # ...######...
        # ####....####
        # #@...##...E#
        # ####....####
        # ...######...
        {
            "player": [(2, 1)],
            "exit": [(2, -2)],
            "walls": [
                rect(1, 1, 1, 3),
                rect(1, -4, 1, -2),
                rect(3, 1, 3, 3),
                rect(3, -4, 3, -2),
                rect(2, 5, 2, 6),
            ],
            "spaces": [
                rect(0, 0, 0, 2),
                rect(-1, 0, -1, 2),
                rect(0, -3, 0, -1),
                rect(-1, -3, -1, -1),
            ],
        },
        # lvl 3 conf
        # ###########
        # #@...#..###
        # ###.#....E#
        # #.....#..##
        # ###########
        {
            "player": [(1, 1)],
            "exit": [(2, -2)],
            "walls": [
                rect(2, 1, 2, 2),
                rect(2, 4, 2, 4),
                rect(1, 5, 1, 5),
                rect(3, 6, 3, 6),
                rect(3, -2, 3, -2),
                rect(1, -3, 1, -2),
            ],
            "spaces": [],
        },
        # lvl 4 conf
        # ......###......
        # #######.#######
        # #@....&.&....E#
        # ###.###.###.###
        # ..#....&....#..
        # ..#####.#####..
        {
            "player": [(2, 1)],
            "exit": [(2, -2)],
            "actiwalls": [
                {
                    "pos": (2, 6),
                    "goto": (1, 7),
                    "plate": Dot({"pos": (2, 5), "dir": "d"}),
                    "retplate": Dot({"pos": (2, 9), "dir": "d"}),
                },
                {
                    "pos": (2, 8),
                    "goto": (3, 7),
                    "plate": Dot({"pos": (2, 5), "dir": "d"}),
                    "retplate": Dot({"pos": (2, 9), "dir": "d"}),
                },
                {
                    "pos": (4, 7),
                    "goto": (5, 7),
                    "plate": Dot({"pos": (4, 8), "dir": "a"}),
                    "retplate": Dot({"pos": (4, 6), "dir": "a"}),
                },
            ],
            "spaces": [
                rect(0, 0, 0, 5),
                rect(0, -6, 0, -1),
                rect(1, 7, 3, 7),
                rect(4, 3, 4, 6),
                rect(4, 8, 4, 11),
                rect(4, 0, 5, 1),
                rect(4, -2, 5, -1),
            ],
            "walls": [
                srect(4, 2),
                srect(4, -3),
                rect(0, 6, 0, 8),
                rect(3, 4, 3, 6),
                rect(3, 8, 3, 10),
                rect(3, 1, 3, 2),
                rect(3, -1, 3, -3),
                rect(1, 1, 1, 6),
                rect(1, 6, 1, -2),
            ],
        },
    ]

    for lvl, conf in enumerate(level_defs):
        for k in ["walls", "spaces"]:
            conf.setdefault(k, [])
        set_chars(out, lvl, conf["player"], player_char)
        set_chars(out, lvl, conf["exit"], exit_char)
        if conf.get("actiwalls"):
            for actiwall_ in conf["actiwalls"]:
                set_chars(out, lvl, actiwall_["pos"], actiwall_char, actiwall=actiwall_)
                set_chars(out, lvl, actiwall_["goto"], space_char)
        for start, end in conf["walls"]:
            fill_rect(out[lvl], start, end, wall_char)
        for start, end in conf["spaces"]:
            fill_rect(out[lvl], start, end, space_char)

    return out


def cout_labyr(map: list[list[str]], chars: dict, clvl: int, entman: EntMan):
    chmap = get_map(chars)

    dir_map = {"w": "↑", "a": "←", "s": "↓", "d": "→"}
    revdir_map = {"w": "⇑", "a": "⇐", "s": "⇓", "d": "⇒"}
    actiwalls: list[Actiwall] = entman.get(clvl, "actiwalls")
    plate_pos: dict = {aw.plate.pos: dir_map.get(aw.plate.dir, "?") for aw in actiwalls}
    retplate_pos: dict = {
        aw.retplate.pos: revdir_map.get(aw.retplate.dir, "¿") for aw in actiwalls
    }

    DARKGRAY = color.C.dark_gray
    print("\n" + DARKGRAY(f"lvl: {clvl}"))
    for y, row in enumerate(map):
        for x, cell in enumerate(row):
            # Handle actiwall plate rendering
            px, py = entman.get(clvl, "player")
            if (y, x) != (py, px):
                if (y, x) in plate_pos:
                    print(DARKGRAY(plate_pos[(y, x)]), end="")
                    continue
                if (y, x) in retplate_pos:
                    print(DARKGRAY(retplate_pos[(y, x)]), end="")
                    continue

            ch = cell[0]

            if ch != "$" and ch in chmap:
                print(chmap[ch](ch), end="")
            elif isinstance(cell, str) and cell.startswith("$M"):
                print(chmap["M"]("M"), end="")
            elif isinstance(cell, tuple):
                print(chmap[ch](ch), end="")
            else:
                print(cell, end="")
        print()


class LabyrGame:
    def __init__(self, *, level=0) -> None:
        c = color.C()
        self.clvl = level
        self.chars = {
            "DEFAULTS": {
                "space": (".", c.dark_gray),
                "wall": ("#", c.dark_gray),
                "actiwall": ("&", c.light_gray),
                "player": ("@", c.blue),
                "exit": ("E", c.green),
                "monster": ("M", c.red),
            },
            "space": (".", c.dark_gray),
            "wall": ("#", c.dark_gray),
            "actiwall": ("&", c.light_gray),
            "player": ("@", c.blue),
            "exit": ("E", c.green),
            "monster": ("M", c.red),
        }
        self.__lvlSizes = {0: (7, 3), 1: (11, 7), 2: (12, 5), 3: (11, 5), 4: (15, 6)}
        self.levels = genlvls(self.chars, self.__lvlSizes)
        self.entman = EntMan(self.levels, self.chars)

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        level = kwds.get("level")
        valid_levels = self.__lvlSizes.keys()
        if isinstance(level, int) and level in valid_levels:
            self.clvl = level
        else:
            if level is not None:
                print(
                    "\033[91mError: 'level must be an integer in {}.\033[0m".format(
                        list(valid_levels)
                    )
                )
                _exit(1)

        running: bool = True
        cmap = self.levels[self.clvl]

        refresh = lambda: (  # noqa: E731
            clsscr(),
            cout_labyr(cmap, self.chars, self.clvl, entman=self.entman),
        )

        win = False

        exit_char = getchar(self.chars, "exit")[0]
        while running:
            if win:
                clsscr()
                print(f"\n{box('YOU WONNNNN')}\n")
                tsleep(Secret())
                _exit(0)
            else:
                refresh()
            move = getch().lower()

            if move in ["w", "a", "s", "d"]:
                out = handlemove(self.chars, self.clvl, cmap, self.entman, move)
                if out == "ESCAPE":
                    refresh = lambda: (  # noqa: E731
                        clsscr(),
                        cout_labyr(cmap, self.chars, self.clvl, entman=self.entman),
                    )

                    nlvl = self.clvl + 1
                    if nlvl > max(self.__lvlSizes.keys()):
                        win = True
                        continue
                    else:
                        plyr = self.entman.get(self.clvl, "player")
                        x, y = plyr
                        refresh()
                        tsleep(0.3)
                        cmap[y][x] = exit_char
                        refresh()
                        tsleep(0.5)
                        self.clvl += 1
                        cmap = self.levels[self.clvl]
            elif move == "r":
                self.levels = genlvls(self.chars, self.__lvlSizes)
                self.entman = EntMan(self.levels, self.chars)
                cmap = self.levels[self.clvl]
                clsscr()
            elif move == "q":
                clsscr()
                _exit(0)


labyr = LabyrGame()



class Dot:
    __slots__ = ("_data",)

    def __init__(self, d: dict | None = None, **kwargs):
        self._data = {}
        if d is not None:
            if isinstance(d, Dot):
                d = d.to_dict()
            for k, v in d.items():
                self[k] = v
        for k, v in kwargs.items():
            self[k] = v

    def _wrap(self, v):
        if isinstance(v, dict):
            return Dot(v)
        return v

    def __getattr__(self, key):
        try:
            return self._data[key]
        except KeyError:
            d = Dot()
            self._data[key] = d
            return d

    def __setattr__(self, key, value):
        if key == "_data":
            super().__setattr__(key, value)
        else:
            self._data[key] = self._wrap(value)

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = self._wrap(value)

    def __contains__(self, key):
        return key in self._data

    def to_dict(self):
        result = {}
        for k, v in self._data.items():
            if isinstance(v, Dot):
                v = v.to_dict()
            result[k] = v
        return result

    def __repr__(self):
        return f"Dot({self.to_dict()})"

    def copy(self):
        return Dot(self.to_dict())

    def merge(self, other):
        for k, v in other.to_dict().items():
            if (
                k in self._data
                and isinstance(self._data[k], Dot)
                and isinstance(v, dict)
            ):
                self._data[k].merge(Dot(v))
            else:
                self._data[k] = self._wrap(v)



def fill_rect(matrix, coord1, coord2, char):
    rows, cols = len(matrix), len(matrix[0])

    def norm(idx, size):
        return idx if idx >= 0 else size + idx

    x1, y1 = norm(coord1[0], rows), norm(coord1[1], cols)
    x2, y2 = norm(coord2[0], rows), norm(coord2[1], cols)
    row_start, row_end = sorted([x1, x2])
    col_start, col_end = sorted([y1, y2])
    for i in range(row_start, row_end + 1):
        for j in range(col_start, col_end + 1):
            matrix[i][j] = char



from dataclasses import dataclass

from . import deltemp, getchar
from .dot import Dot


@dataclass
class Player:
    x: int
    y: int

    def __iter__(self):
        yield self.x
        yield self.y


@dataclass
class Actiwall:
    pos: tuple[
        int, int
    ]  # original position also to return to when return pressure plate activated
    goto: tuple[int, int]
    plate: Dot
    retplate: Dot
    activated: bool = False


class EntMan:
    """entity manager"""

    def __init__(self, levels, chars) -> None:
        self.lvls = levels
        self._entities = {}

        def retTable(matrix: list[list[str | tuple]]):
            out = {}
            player_char = getchar(chars, "player")[0]
            actiwall_char = getchar(chars, "actiwall")[0]
            actiwalls: list[Actiwall] = []
            for y, row in enumerate(matrix):
                for x, char in enumerate(row):
                    if char == player_char:
                        out["player"] = Player(x, y)
                    elif isinstance(char, tuple) and char[0] == actiwall_char:
                        actiwalls.append(char[1])
                    else:
                        print(char)
            out["actiwalls"] = actiwalls
            return out

        for lvl, lmap in levels.items():
            self._entities[lvl] = retTable(lmap)

    def get(self, lvl: int, entity: str):
        return self._entities[lvl][entity]


def handlemove(chars, clvl, cmap: list, entman: EntMan, move: str):
    x, y = entman.get(clvl, "player")
    move_offsets = {"w": (0, -1), "a": (-1, 0), "s": (0, 1), "d": (1, 0)}
    dx, dy = move_offsets.get(move, (0, 0))
    nx, ny = x + dx, y + dy

    if not (0 <= ny < len(cmap) and 0 <= nx < len(cmap[0])):
        return

    player_char = getchar(chars, "player")[0]
    space_char = getchar(chars, "space")[0]
    exit_char = getchar(chars, "exit")[0]
    actiwall_char = getchar(chars, "actiwall")[0]
    target = cmap[ny][nx]

    if target in (space_char, exit_char):
        cmap[y][x] = space_char
        cmap[ny][nx] = player_char
        entman._entities[clvl]["player"].x, entman._entities[clvl]["player"].y = nx, ny
        if target == exit_char:
            return "ESCAPE"

    for actiwall in entman.get(clvl, "actiwalls"):
        _gty, _gtx = actiwall.goto
        _posy, _posx = actiwall.pos
        if (  # activate actiwall
            (ny, nx) == actiwall.plate.pos
            and move == actiwall.plate.dir
            and not actiwall.activated
        ):
            actiwall.activated = True
            cmap[_gty][_gtx] = actiwall_char
            cmap[_posy][_posx] = space_char
            deltemp()
        elif (
            (ny, nx) == actiwall.retplate.pos
            and move == actiwall.retplate.dir
            and actiwall.activated
        ):
            actiwall.activated = False
            cmap[_gty][_gtx] = space_char
            cmap[_posy][_posx] = actiwall_char



from typing import Any


class _Secret:
    def __init__(self) -> None:
        self._idx = 0
        self._secret = []

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        self._idx += 1
        return self._secret[self._idx - 1]

    def extend(self, other):
        self._secret.append(other)

    def pop(self, other):
        self._idx = max(0, self._idx - other)


Secret = _Secret()


def _1():
    _0x1a = lambda x: __import__("math").tan(x) ** 2 * 0.000001  # noqa: E731
    _0x2b = lambda y: (y ^ 42) * 0.00001  # noqa: E731
    num = (
        __import__("base64").b64decode("MQ==").decode()
        + chr(46)
        + "".join(
            [
                __import__("base64").b64decode("MDQy").decode(),
                __import__("base64").b64decode("MDY5").decode(),
            ]
        )
    )
    __0x3c = lambda z: "".join(chr((ord(c) ^ 3) + 2) for c in z)  # noqa: E731
    return float(num) - _0x1a(__import__("math").pi / 4) - _0x2b(0x2A) + 1e-6


Secret.extend(_1())



from typing import Callable, Iterable


def get_map(d: dict):
    return {v[0]: v[1] for k, v in d.items() if not k.isupper()}


def transform(args: Iterable, f: Callable) -> list:
    return list(map(f, args))
